#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for anonip.
"""


from __future__ import print_function, unicode_literals

import argparse
import logging
import re
import sys
from io import StringIO

import pytest

import anonip


@pytest.mark.parametrize(
    "ip,v4mask,v6mask,expected",
    [
        ("192.168.100.200", 12, 84, "192.168.96.0"),
        ("192.168.100.200:80", 12, 84, "192.168.96.0:80"),
        ("192.168.100.200]", 12, 84, "192.168.96.0]"),
        ("192.168.100.200:80]", 12, 84, "192.168.96.0:80]"),
        ("192.168.100.200", 0, 84, "192.168.100.200"),
        ("192.168.100.200", 4, 84, "192.168.100.192"),
        ("192.168.100.200", 8, 84, "192.168.100.0"),
        ("192.168.100.200", 24, 84, "192.0.0.0"),
        ("192.168.100.200", 32, 84, "0.0.0.0"),
        ("no_ip_address", 12, 84, "no_ip_address"),
        ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 12, 84, "2001:db8:85a0::"),
        (
            "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:443",
            12,
            84,
            "[2001:db8:85a0::]:443",
        ),
        ("[2001:0db8:85a3:0000:0000:8a2e:0370:7334]", 12, 84, "[2001:db8:85a0::]"),
        ("[2001:0db8:85a3:0000:0000:8a2e:0370:7334]]", 12, 84, "[2001:db8:85a0::]]"),
        (
            "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:443]",
            12,
            84,
            "[2001:db8:85a0::]:443]",
        ),
        (
            "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
            12,
            0,
            "2001:db8:85a3::8a2e:370:7334",
        ),
        (
            "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
            12,
            4,
            "2001:db8:85a3::8a2e:370:7330",
        ),
        (
            "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
            12,
            8,
            "2001:db8:85a3::8a2e:370:7300",
        ),
        (
            "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
            12,
            24,
            "2001:db8:85a3::8a2e:300:0",
        ),
        ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 12, 32, "2001:db8:85a3::8a2e:0:0"),
        ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 12, 62, "2001:db8:85a3::"),
        ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", 12, 128, "::"),
        ("   foo", 12, 84, "   foo"),
    ],
)
def test_process_line(ip, v4mask, v6mask, expected):
    a = anonip.Anonip(ipv4mask=v4mask, ipv6mask=v6mask)
    assert a.process_line(ip) == expected


@pytest.mark.parametrize(
    "ip,increment,expected",
    [
        ("192.168.100.200", 3, "192.168.96.3"),
        ("192.168.100.200", 284414028745874325, "192.168.96.0"),
    ],
)
def test_increment(ip, increment, expected):
    a = anonip.Anonip(increment=increment)
    assert a.process_line(ip) == expected


@pytest.mark.parametrize(
    "line,columns,expected",
    [
        (
            "192.168.100.200 some string with öéäü",
            None,
            "192.168.96.0 some string with öéäü",
        ),
        (
            "some 192.168.100.200 string with öéäü",
            [2],
            "some 192.168.96.0 string with öéäü",
        ),
        (
            "some string 192.168.100.200 with öéäü",
            [3],
            "some string 192.168.96.0 with öéäü",
        ),
        (
            "192.168.100.200 192.168.11.222 192.168.123.234",
            [1, 2, 3],
            "192.168.96.0 192.168.0.0 192.168.112.0",
        ),
        (
            "192.168.100.200 192.168.11.222 192.168.123.234",
            [9999],
            "192.168.100.200 192.168.11.222 192.168.123.234",
        ),
    ],
)
def test_column(line, columns, expected):
    a = anonip.Anonip(columns=columns)
    assert a.process_line(line) == expected


@pytest.mark.parametrize(
    "line,regex,expected,replace",
    [
        (
            '3.3.3.3 - - [20/May/2015:21:05:01 +0000] "GET / HTTP/1.1" 200 13358 "-" "useragent"',
            re.compile(
                r"(?:^([^,]+) - - |.* - somefixedstring: ([^,]+) - .* - ([^,]+))"
            ),
            '3.3.0.0 - - [20/May/2015:21:05:01 +0000] "GET / HTTP/1.1" 200 13358 "-" "useragent"',
            None,
        ),
        (
            "blabla/ 3.3.3.3 /blublu",
            re.compile(r"^blabla/ ([^,]+) /blublu"),
            "blabla/ 3.3.0.0 /blublu",
            None,
        ),
        (
            "1.1.1.1 - somefixedstring: 2.2.2.2 - some random stuff - 3.3.3.3",
            re.compile(r"^([^,]+) - somefixedstring: ([^,]+) - .* - ([^,]+)"),
            "1.1.0.0 - somefixedstring: 2.2.0.0 - some random stuff - 3.3.0.0",
            None,
        ),
        (
            "some line that doesn't match the provided regex",
            re.compile(r"^([^,]+) - somefixedstring: ([^,]+) - .* - ([^,]+)"),
            "some line that doesn't match the provided regex",
            None,
        ),
        (
            "match but no ip/ notanip /blublu",
            re.compile(r"^match but no ip/ ([^,]+) /blublu"),
            "match but no ip/ notanip /blublu",
            None,
        ),
        (
            "match but no ip/ notanip /blublu",
            re.compile(r"^match but no ip/ ([^,]+) /blublu"),
            "match but no ip/ yeah /blublu",
            "yeah",
        ),
    ],
)
def test_regex(line, regex, expected, replace):
    a = anonip.Anonip(regex=regex, replace=replace)
    assert a.process_line(line) == expected


def test_replace():
    a = anonip.Anonip(replace="replacement")
    assert a.process_line("bla something") == "replacement something"


def test_delimiter():
    a = anonip.Anonip(delimiter=";")
    assert (
        a.process_line("192.168.100.200;some;string;with;öéäü")
        == "192.168.96.0;some;string;with;öéäü"
    )


def test_private():
    a = anonip.Anonip(skip_private=True)
    assert a.process_line("192.168.100.200") == "192.168.100.200"


def test_run(monkeypatch):
    a = anonip.Anonip()

    monkeypatch.setattr(
        "sys.stdin", StringIO("192.168.100.200\n1.2.3.4\n  \n9.8.130.6\n")
    )

    lines = [line for line in a.run()]
    assert lines == ["192.168.96.0", "1.2.0.0", "", "9.8.128.0"]


def test_run_with_input_file():
    a = anonip.Anonip()

    input_file = StringIO("192.168.100.200\n1.2.3.4\n  \n9.8.130.6\n")

    lines = [line for line in a.run(input_file)]
    assert lines == ["192.168.96.0", "1.2.0.0", "", "9.8.128.0"]


@pytest.mark.parametrize(
    "args,attribute,expected",
    [
        (["-c", "3", "5"], "columns", [3, 5]),
        (["-4", "24"], "ipv4mask", 24),
        (["-6", "64"], "ipv6mask", 64),
    ],
)
def test_cli_generic_args(args, attribute, expected):
    assert getattr(anonip.parse_arguments(args), attribute) == expected


@pytest.mark.parametrize(
    "args,success",
    [
        ([], True),
        (["--regex", "test"], True),
        (["-c", "4"], True),
        (["--regex", "test", "-c", "3"], False),
        (["--regex", "test", "-l", ";"], False),
        (["--regex", "test", "-l", ";", "-c", "4"], False),
    ],
)
def test_cli_args_ambiguity(args, success):
    if success:
        anonip.parse_arguments(args)
        return

    with pytest.raises(SystemExit) as e:
        anonip.parse_arguments(args)
    assert e.value.code == 2


@pytest.mark.parametrize(
    "args,expected",
    [
        (["--regex", "test"], "test"),
        (["--regex", "foo", "bar", "baz"], "foo|bar|baz"),
    ],
)
def test_regex_concat(args, expected):
    args = anonip.parse_arguments(args)
    assert args.regex == re.compile(expected)


@pytest.mark.parametrize(
    "value,valid,bits",
    [
        ("1", True, 32),
        ("0", False, 32),
        ("33", False, 32),
        ("string", False, 32),
        ("129", False, 128),
    ],
)
def test_cli_validate_ipmask(value, valid, bits):
    if valid:
        assert anonip._validate_ipmask(value, bits) == int(value)
    else:
        with pytest.raises(argparse.ArgumentTypeError):
            anonip._validate_ipmask(value, bits)


@pytest.mark.parametrize(
    "value,valid", [("1", True), ("0", False), ("-1", False), ("string", False)]
)
def test_cli_validate_integer_ht_0(value, valid):
    if valid:
        assert anonip._validate_integer_ht_0(value) == int(value)
    else:
        with pytest.raises(argparse.ArgumentTypeError):
            anonip._validate_integer_ht_0(value)


@pytest.mark.parametrize("value,valid", [("valid (.*)", True), ("\\9", False)])
def test_regex_arg_type(value, valid):
    if valid:
        assert anonip.regex_arg_type(value) == value
    else:
        with pytest.raises(argparse.ArgumentTypeError):
            anonip.regex_arg_type(value)


@pytest.mark.parametrize("to_file", [False, True])
@pytest.mark.parametrize("debug,log_level", [(False, 30), (True, 10)])
def test_main(
    to_file,
    debug,
    log_level,
    backup_and_restore_sys_argv,
    capsys,
    monkeypatch,
    tmp_path,
):
    log_file = tmp_path / "anonip.log"
    sys.argv = [
        "anonip.py",
        "-c",
        "2",
        "-4",
        "12",
        "-6",
        "42",
        "-i",
        "1",
        "-l",
        ";",
        "-r",
        "replace",
        "-p",
    ]
    if to_file:
        sys.argv += ["-o", str(log_file)]
    if debug:
        sys.argv.append("-d")

    monkeypatch.setattr(
        "sys.stdin",
        StringIO(
            "string;192.168.100.200\n"
            "string;1.2.3.4\n"
            "string;2001:0db8:85a3:0000:0000:8a2e:0370:7334\n"
            "string;2a00:1450:400a:803::200e\n"
            "string;string\n"
        ),
    )
    anonip.main()

    if to_file:
        with log_file.open() as f:
            lines = [line.rstrip("\n") for line in f.readlines()]
    else:
        captured = capsys.readouterr()
        lines = captured.out.split("\n")[:-1]

    assert lines == [
        "string;192.168.100.200",
        "string;1.2.0.1",
        "string;2001:db8:85a3::8a2e:370:7334",
        "string;2a00:1450:400a:803::1",
        "string;replace",
    ]

    logger = logging.getLogger("anonip")
    assert logger.level == log_level


def test_main_reading_from_input_file(tmp_path, capsys, backup_and_restore_sys_argv):
    input_filename = tmp_path / "anonip-input.txt"
    input_filename.write_text(
        "192.168.100.200 string\n"
        "1.2.3.4 string\n"
        "2001:0db8:85a3:0000:0000:8a2e:0370:7334 string\n"
        "2a00:1450:400a:803::200e string\n"
    )
    sys.argv = ["anonip.py", "--input", str(input_filename), "-d"]
    anonip.main()
    captured = capsys.readouterr()
    lines = captured.out.split("\n")[:-1]
    assert lines == [
        "192.168.96.0 string",
        "1.2.0.0 string",
        "2001:db8:85a0:: string",
        "2a00:1450:4000:: string",
    ]


def test_prefixes_dict():
    a = anonip.Anonip(ipv4mask=11, ipv6mask=83)
    prefixes = a._prefixes
    assert len(prefixes) == 2
    assert 4 in prefixes and bool(prefixes[4])
    assert 6 in prefixes and bool(prefixes[6])


def test_properties_v4():
    a = anonip.Anonip(ipv4mask=11, ipv6mask=83)
    assert a.ipv4mask == 11
    assert a._prefixes[4] == 21


def test_properties_v6():
    a = anonip.Anonip(ipv4mask=11, ipv6mask=83)
    assert a.ipv6mask == 83
    assert a._prefixes[6] == 45


def test_properties_columns():
    a = anonip.Anonip()
    assert a.columns == [0]
    a.columns = [5, 6]
    assert a.columns == [4, 5]



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
An IP address anonymizer
Digitale Gesellschaft
https://www.digitale-gesellschaft.ch.
Special thanks to: Thomas B. and Fabio R.

Copyright (c) 2013 - 2016, Swiss Privacy Foundation
              2016 - 2019, Digitale Gesellschaft
              2019,        Hartmut Goebel
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
 * Neither the name of copyright holder nor the names of its
   contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
"""

from __future__ import print_function, unicode_literals

import argparse
import logging
import re
import sys
from io import open

try:
    import ipaddress
except ImportError:  # pragma: no cover
    # Could happen with python < 3.3
    print("\033[31;1mError: Module ipaddress not found.\033[0m", file=sys.stderr)
    sys.exit(1)
try:
    from urllib.parse import urlparse
except ImportError:  # pragma: no cover
    # compatibility for python < 3
    from urlparse import urlparse

if sys.version_info[0] >= 3:  # pragma: no cover
    # compatibility for python < 3
    unicode = str

__title__ = "anonip"
__description__ = "Anonip is a tool to anonymize IP-addresses in log files."
__version__ = "1.1.0"
__license__ = "BSD"
__author__ = "Digitale Gesellschaft"

logger = logging.getLogger(__name__)


class Anonip(object):
    def __init__(
        self,
        columns=None,
        ipv4mask=12,
        ipv6mask=84,
        increment=0,
        delimiter=" ",
        replace=None,
        regex=None,
        skip_private=False,
    ):
        """
        Main class for anonip.

        :param columns: list of int, 1-based column numbers
        :param ipv4mask: int
        :param ipv6mask: int
        :param increment: int
        :param delimiter: str
        :param replace: str
        :param skip_private: bool
        """
        self.columns = columns
        self._prefixes = {}  # next two lines will fill the values
        self.ipv4mask = ipv4mask
        self.ipv6mask = ipv6mask
        self.increment = increment
        self.delimiter = delimiter
        self.replace = replace
        self.regex = regex
        self.skip_private = skip_private

    @property
    def columns(self):
        return self._columns

    @columns.setter
    def columns(self, columns):
        # change columns to be 0-based
        self._columns = [c - 1 for c in columns] if columns else [0]

    @property
    def ipv4mask(self):
        return self._ipv4mask

    @ipv4mask.setter
    def ipv4mask(self, mask):
        self._ipv4mask = mask
        self._prefixes[4] = 32 - mask

    @property
    def ipv6mask(self):
        return self._ipv6mask

    @ipv6mask.setter
    def ipv6mask(self, mask):
        self._ipv6mask = mask
        self._prefixes[6] = 128 - mask

    def run(self, input_file=None):
        """
        Generator that reads from file handle (defaults to stdin)
        and loops forever.

        Yields anonymized log lines.

        :param input_file: file handle to read from (default: sys.stdin)
        :return: None
        """
        if not input_file:
            # Assign here instead of using a default parameter value
            # to allow "late binding".
            input_file = sys.stdin
        line = input_file.readline()
        while line:
            line = line.rstrip()

            if line.strip() == "":
                logger.debug("Empty line detected. Doing nothing.")
                yield line
                line = input_file.readline()
                continue

            logger.debug("Got line: %r", line)

            yield self.process_line(line)

            line = input_file.readline()

    def process_ip(self, ip):
        """
        Process a single ip.

        :param ip: /32 ipaddress.IPv4Network or /128 ipaddress.IPv6Network
        :return: ipaddress.IPv4Address or ipaddress.IPv6Address
        """
        if self.skip_private and ip[0].is_private:
            return ip[0]
        else:
            trunc_ip = self.truncate_address(ip)
            if self.increment:
                try:
                    trunc_ip = trunc_ip + self.increment
                except ipaddress.AddressValueError:
                    logger.error(
                        "Could not increment IP %s by %s", trunc_ip, self.increment
                    )
            return trunc_ip

    def process_line_regex(self, line):
        """
        This function processes a single line based on the provided regex.

        It returns the anonymized log line as string.

        :param line: str
        :return: str
        """
        match = re.match(self.regex, line)
        if not match:
            logger.debug("Regex did not match!")
            return line
        groups = match.groups()

        for m in set(groups):
            if not m:
                continue
            ip_str, ip = self.extract_ip(m)
            if ip:
                trunc_ip = self.process_ip(ip)
                line = line.replace(ip_str, str(trunc_ip))
            elif self.replace:
                line = line.replace(m, self.replace)

        return line

    def process_line_column(self, line):
        """
        This function processes a single line based on the provided columns.

        It returns the anonymized log line as string.

        :param line: str
        :return: str
        """
        loglist = line.split(self.delimiter)

        for index in self.columns:
            try:
                column = loglist[index]
            except IndexError:
                logger.warning("Column %s does not exist!", index + 1)
                continue
            else:
                if column == "":
                    logger.debug("Column %s is empty.", index + 1)
                    continue
                ip_str, ip = self.extract_ip(column)
                if ip:
                    trunc_ip = self.process_ip(ip)
                    loglist[index] = column.replace(ip_str, str(trunc_ip))
                elif self.replace:
                    loglist[index] = self.replace

        return self.delimiter.join(loglist)

    def process_line(self, line):
        """
        This function processes a single line.
        It returns the anonymized log line as string.

        :param line: str
        :return: str
        """
        if self.regex:
            return self.process_line_regex(line)
        return self.process_line_column(line)

    @staticmethod
    def extract_ip(column):
        """
        This function extracts the ip from the column and returns it.

        It can handle following ip formats:
         - 192.168.100.200
         - 192.168.100.200:80
         - 192.168.100.200]
         - 192.168.100.200:80]
         - 2001:0db8:85a3:0000:0000:8a2e:0370:7334
         - [2001:0db8:85a3:0000:0000:8a2e:0370:7334]
         - [2001:0db8:85a3:0000:0000:8a2e:0370:7334]]
         - [2001:0db8:85a3:0000:0000:8a2e:0370:7334]:443
         - [2001:0db8:85a3:0000:0000:8a2e:0370:7334]:443]

        :param column: str
        :return: tuple (
                ip str,
                /32 ipaddress.IPv4Network or /128 ipaddress.IPv6Network) or
                (None, None)
        """

        # first we try if the whole column is just the ip
        try:
            ip = ipaddress.ip_network(unicode(column))
            return column, ip
        except ValueError:
            # then we try if the ip has the port appended and/or a trailing ']'
            try:
                # strip additional ']' from column. Ugly but functional
                if (column.startswith("[") and column.endswith("]]")) or (
                    not column.startswith("[") and column.endswith("]")
                ):
                    column = column[:-1]

                parsed = urlparse("//{}".format(column))
                new_column = parsed.hostname
                ip = ipaddress.ip_network(unicode(new_column))
                return new_column, ip
            except Exception as e:
                logger.warning(e)
                return None, None

    def truncate_address(self, ip):
        """
        Do the actual masking of the IP address

        :param ip: ipaddress object
        :return: ipaddress object
        """
        return ip.supernet(new_prefix=self._prefixes[ip.version])[0]


def _validate_ipmask(mask, bits=32):
    """
    Verify if the supplied ip mask is valid.

    :param mask: the provided ip mask
    :param bits: 32 for ipv4, 128 for ipv6
    :return: int
    """
    msg = "must be an integer between 1 and {}".format(bits)
    try:
        mask = int(mask)
    except ValueError:
        raise argparse.ArgumentTypeError(msg)

    if not 0 < mask <= bits:
        raise argparse.ArgumentTypeError(msg)

    return mask


def _validate_integer_ht_0(value):
    """
    Validate if given string is a number higher than 0.

    :param value: str or int
    :return: int
    """
    msg = "must be a positive integer"
    try:
        value = int(value)
    except ValueError:
        raise argparse.ArgumentTypeError(msg)
    if not value >= 1:
        raise argparse.ArgumentTypeError(msg)
    return value


def regex_arg_type(value):
    try:
        re.compile(value)
    except re.error as e:
        msg = "must be a valid regex."
        if hasattr(e, "msg"):  # pragma: no cover
            # not available on py27
            msg = "must be a valid regex. Error: {}".format(e.msg)
        raise argparse.ArgumentTypeError(msg)
    return value


def parse_arguments(args):
    """
    Parse all given arguments.

    :param args: list
    :return: argparse.Namespace
    """
    parser = argparse.ArgumentParser(
        description=__description__,
        epilog="Example-usage in apache-config:\n"
        'CustomLog "| /path/to/anonip.py '
        '[OPTIONS] --output /path/to/log" '
        "combined\n ",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "-4",
        "--ipv4mask",
        metavar="INTEGER",
        help="truncate the last n bits (default: %(default)s)",
        type=lambda x: _validate_ipmask(x, 32),
    )
    parser.set_defaults(ipv4mask=12)
    parser.add_argument(
        "-6",
        "--ipv6mask",
        type=lambda x: _validate_ipmask(x, 128),
        metavar="INTEGER",
        help="truncate the last n bits (default: %(default)s)",
    )
    parser.set_defaults(ipv6mask=84)
    parser.add_argument(
        "-i",
        "--increment",
        metavar="INTEGER",
        type=lambda x: _validate_integer_ht_0(x),
        help="increment the IP address by n (default: %(default)s)",
    )
    parser.set_defaults(increment=0)
    parser.add_argument("-o", "--output", metavar="FILE", help="file to write to")
    parser.add_argument(
        "--input", metavar="FILE", help="File or FIFO to read from (default: stdin)"
    )
    parser.add_argument(
        "-c",
        "--column",
        metavar="INTEGER",
        dest="columns",
        nargs="+",
        type=lambda x: _validate_integer_ht_0(x),
        help="assume IP address is in column n (1-based indexed; default: 1)",
    )
    parser.add_argument(
        "-l",
        "--delimiter",
        metavar="STRING",
        type=str,
        help='log delimiter (default: " ")',
    )
    parser.add_argument(
        "--regex",
        metavar="STRING",
        nargs="+",
        help="regex for detecting IP addresses (use optionally instead of -c)",
        type=regex_arg_type,
    )
    parser.add_argument(
        "-r",
        "--replace",
        metavar="STRING",
        help="replacement string in case address parsing fails (Example: 0.0.0.0)",
    )
    parser.add_argument(
        "-p",
        "--skip-private",
        dest="skip_private",
        action="store_true",
        help="do not mask addresses in private ranges. "
        "See IANA Special-Purpose Address Registry.",
    )
    parser.add_argument(
        "-d", "--debug", action="store_true", help="print debug messages"
    )
    parser.add_argument("-v", "--version", action="version", version=__version__)

    args = parser.parse_args(args)

    if args.regex and (args.columns is not None or args.delimiter is not None):
        raise parser.error(
            'Ambiguous arguments: When using "--regex", "-c" and "-l" can\'t be used.'
        )
    if not args.regex and args.columns is None:
        args.columns = [1]
    if not args.regex and args.delimiter is None:
        args.delimiter = " "
    if args.regex:
        try:
            args.regex = re.compile(r"|".join(args.regex))
        except re.error:  # pragma: no cover
            raise argparse.ArgumentTypeError("Failed to compile concatenated regex!")

    return args


def main():
    """
    Main CLI function for anonip.
    """
    args = parse_arguments(sys.argv[1:])

    logging.basicConfig()
    if args.debug:
        logger.level = logging.DEBUG
    else:
        logger.level = logging.WARNING

    anonip = Anonip(
        args.columns,
        args.ipv4mask,
        args.ipv6mask,
        args.increment,
        args.delimiter,
        args.replace,
        args.regex,
        args.skip_private,
    )

    input_file = output_file = None
    try:
        if args.input:
            input_file = open(args.input, "r")
        if args.output:
            output_file = open(args.output, "a")
        else:
            output_file = sys.stdout
        for line in anonip.run(input_file):
            print(unicode(line), file=output_file)
            # TODO: when dropping support for Python <= 3.3, move the
            # flush into the print()
            output_file.flush()
    except IOError as err:  # pragma: no cover
        logger.error(err)
    except KeyboardInterrupt:  # pragma: no cover
        pass
    finally:
        if args.input and input_file:
            input_file.close()
        if args.output and output_file:
            output_file.close()


if __name__ == "__main__":  # pragma: no cover
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
setup.py for anonip
"""

from io import open
from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="anonip",
    version="1.1.0",
    description="Anonip is a tool to anonymize IP-addresses in log-files.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DigitaleGesellschaft/Anonip",
    author="Digitale Gesellschaft",
    license="BSD",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    install_requires=['ipaddress; python_version<"3.3"'],
    py_modules=["anonip"],
    entry_points={"console_scripts": ["anonip = anonip:main"]},
)
